# Hunter's Skin Chooser Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/skin_chooser/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load skin_chooser
#
################################################################################
#import EventScripts
import es
#import Libraries
import services
import popuplib
import playerlib
import gamethread
import os
import random
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Skin Chooser"
info.version = "2.0.0"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Allows players and admins to choose special skins"
info.basename = "skin_chooser"

hunter_skin_chooser_ver    = info.version
hunter_skin_chooser_text   = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Server Variables
skin_chooser_download      = es.ServerVar('skin_chooser_download', '1', 'Should the files be downloaded with EventScripts?')
skin_chooser_botskins      = es.ServerVar('skin_chooser_botskins', '1', 'Should bots get random public skins?')

# Global Variables
skin_chooser_downloadables = []
skin_chooser_skins         = []
skin_chooser_players       = {}
skin_chooser_popups        = {}

def load():
    public = es.ServerVar('hu_sc', info.version, info.name)
    public.makepublic()
    
    auth = services.use('auth')
    auth.registerCapability('set_skin', auth.ADMIN)
    auth.registerCapability('admin_skins', auth.ADMIN)
    auth.registerCapability('reserved_skins', auth.POWERUSER)

    es.regcmd('skin_chooser_addskin', 'skin_chooser/addskin', 'Add a type based skin to Hostage Skins, valid types are normal_t, normal_ct, reserved_t, reserved_ct, admin_t, admin_ct')
    es.regcmd('skin_chooser_adddownload', 'skin_chooser/adddownload', 'Add files to Hostage Skins auto-download')
    es.regsaycmd('!skin', 'skin_chooser/skin', 'Skin Chooser Settings')
    es.regsaycmd('!setskin', 'skin_chooser/setskin', 'Skin Chooser Admin')

    es.log(hunter_skin_chooser_text)
    es.msg('#multi', '#green[SkinChooser] #defaultLoaded')
    
def unload():
    for popup in skin_chooser_popups:
        skin_chooser_popups[popup].delete()

    es.unregsaycmd('!skin')
    es.unregsaycmd('!setskin')
    es.msg('#multi', '#green[SkinChooser] #defaultUnloaded') 

def es_map_start(event_var):
    if int(skin_chooser_download):
        for filename in skin_chooser_downloadables:
            es.stringtable('downloadables', filename)
    for skin in skin_chooser_skins:
        es.precachemodel(_mdlstr(skin['model']))

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_skin_chooser_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_skin_chooser_text)

def player_activate(event_var):
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_skin_chooser_text))

def player_spawn(event_var):
    player = playerlib.getPlayer(event_var['userid'])
    if player and skin_chooser_skins:
        if int(player) in skin_chooser_players and int(player.attributes['teamid']) > 1:
            player.set('model', _mdlstr(skin_chooser_players[int(player)]))
        elif int(skin_chooser_botskins) and int(player.attributes['isbot']) == 1:
            if int(player.attributes['teamid']) == 2:
                player.set('model', _mdlstr(random.choice(filter(lambda x: x['type'] == 'normal_t', skin_chooser_skins))['model']))
            elif int(player.attributes['teamid']) == 3:
                player.set('model', _mdlstr(random.choice(filter(lambda x: x['type'] == 'normal_ct', skin_chooser_skins))['model']))

def player_disconnect(event_var):
    if skin_chooser_skins:
        if int(event_var['userid']) in skin_chooser_players:
            del skin_chooser_players[int(event_var['userid'])]
        if int(event_var['userid']) in skin_chooser_popups:
            del skin_chooser_popups[int(event_var['userid'])]

def addskin():
    if int(es.getargc()) >= 3:
        skin = {'type':es.getargv(1), 'name':es.getargv(2), 'model':es.getargv(3)}
        skin_chooser_skins.append(skin)
    else:
        es.dbgmsg(0, 'Syntax: skin_chooser_addskin <type> <name> <model>')

def adddownload():
    if int(es.getargc()) == 2:
        if os.path.exists(str(es.server_var['eventscripts_gamedir'])+'/'+str(es.getargv(1))):
            skin_chooser_downloadables.append(str(es.getargv(1)))
            es.stringtable('downloadables', str(es.getargv(1)))
        else:
            es.dbgmsg(0, 'Could not add downloadable '+str(es.getargv(1))+', file not found!')
    else:
        es.dbgmsg(0, 'Syntax: skin_chooser_adddownload <file>')

def skin():
    player = playerlib.getPlayer(es.getcmduserid())
    if player:
        auth = services.use('auth')
        skin_chooser_popups[int(player)] = popuplib.easymenu('skin_chooser_%s'%int(player), None, _select_skin)
        skin_chooser_popups[int(player)]._target = player
        skin_chooser_popups[int(player)].settitle('Skin Chooser')
        skin_chooser_popups[int(player)].vguititle = 'Skin Chooser'
        skin_chooser_popups[int(player)].addoption('default', 'Default')
        for skin in skin_chooser_skins:
            if int(player.attributes['teamid']) == 2 and skin['type'].endswith('_t') or int(player.attributes['teamid']) == 3 and skin['type'].endswith('_ct'):
                if skin['type'].startswith('normal_'):
                    skin_chooser_popups[int(player)].addoption(skin['type']+skin['name'].lower(), skin['name'])
                elif skin['type'].startswith('reserved_') and auth.isUseridAuthorized(int(player), 'reserved_skins'):
                    skin_chooser_popups[int(player)].addoption(skin['type']+skin['name'].lower(), skin['name'])
                elif skin['type'].startswith('admin_') and auth.isUseridAuthorized(int(player), 'admin_skins'):
                    skin_chooser_popups[int(player)].addoption(skin['type']+skin['name'].lower(), skin['name'])
        skin_chooser_popups[int(player)].send(int(player))

def setskin():
    if int(es.getargc()) == 2 and es.getuserid(es.getargv(1)):
        player = playerlib.getPlayer(es.getcmduserid())
        target = playerlib.getPlayer(es.getuserid(es.getargv(1)))
        if player and target:
            auth = services.use('auth')
            if auth.isUseridAuthorized(int(player), 'set_skin'):
                skin_chooser_popups[0] = popuplib.easymenu("skin_chooser_admin", None, _select_skin)
                skin_chooser_popups[0]._target = target
                skin_chooser_popups[0].settitle('Set %s\'s Skin' % target.attributes['name'])
                skin_chooser_popups[0].vguititle = 'Set %s\'s Skin' % target.attributes['name']
                skin_chooser_popups[0].addoption('default', 'Default')
                for skin in skin_chooser_skins:
                    skin_chooser_popups[0].addoption(skin['type']+skin['name'].lower(), skin['name'])
                skin_chooser_popups[0].send(int(player))
            else:
                es.tell(es.getcmduserid(), '#multi', '#green[SkinChooser] #lightgreenNo permission!')
        else:
            es.tell(es.getcmduserid(), '#multi', '#green[SkinChooser] #lightgreenThis user was not found!')
    else:
        es.tell(es.getcmduserid(), '#multi', '#green[SkinChooser] #lightgreenYou need to specify a user!')

def _select_skin(userid, choice, popupid):
    player = playerlib.getPlayer(userid)
    popup = popuplib.find(popupid)
    if player and popup and popup._target:
        if choice == 'default':
            if int(popup._target) in skin_chooser_players:
                del skin_chooser_players[int(popup._target)]
        else:
            for skin in skin_chooser_skins:
                if skin['type']+skin['name'].lower() == choice:
                    skin_chooser_players[int(popup._target)] = skin['model']

def _mdlstr(model):
    if model[:7] != 'models/':
        model = 'models/'+model
    if model[-4:] != '.mdl':
        model = model + '.mdl'
    return model
